from shrinkit.main import *
# import os
# import sys
# import streamlit.web.cli as stcli


# this_dir = os.path.dirname(os.path.abspath(__file__))
# os.chdir(this_dir)
# print(this_dir)
# sys.path.append(this_dir)
# sys.argv = ["streamlit", "run", "main.py", "--global.developmentMode=false", "--browser.gatherUsageStats=false"]
# sys.exit(stcli.main())